/*-
 * Copyright (c) 2012 HATANO Tomomi.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * panalcdsyncd.c
 *	Synchronize LCD brightness values between
 *	acpi_panasonic(4) and acpi_video(4).
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/sysctl.h>

#define	SYSCTL_ACPI_VIDEO	"hw.acpi.video.lcd0.brightness"
#define	SYSCTL_ACPI_PANASONIC	"hw.acpi.panasonic.lcd_brightness"

void
usage(char *me)
{
	fprintf(stderr, "Usage: %s [-d] [-i interval]\n", me);
}

int
sync_lcd_brightness()
{
	int r, v, p, pv;
	size_t vlen, plen;

	/* Get current values. */
	plen = sizeof(p);
	r = sysctlbyname(SYSCTL_ACPI_PANASONIC, &p, &plen, NULL, 0);
	if (r != 0) {	/* Error. */
		return r;
	}
	vlen = sizeof(v);
	r = sysctlbyname(SYSCTL_ACPI_VIDEO, &v, &vlen, NULL, 0);
	if (r != 0) {	/* Error. */
		return r;
	}

	/* Compare. */
	pv = (p - 1) * 5;	/* panasonic: 1-21, video: 0-100 */
	if (pv != v) {
		/* Differ. Write new value. */
		r = sysctlbyname(SYSCTL_ACPI_VIDEO, &p, &plen,
		    &pv, sizeof(pv));
	}

	return r;
}

int
main(int argc, char *argv[])
{
	int ch;
	int interval = 2;	/* Default interval: 2 secs. */
	int dflg = 0;		/* No daemonize by default. */
	char *me;

	me = argv[0];
	while ((ch = getopt(argc, argv, "dhi:")) != -1) {
		switch (ch) {
		case 'd':
			dflg = 1;
			break;
		case 'i':
			interval = atoi(optarg);
			break;
		case 'h':
		case '?':
			usage(me);
			exit(1);
		}
	}
	argc -= optind;
	argv += optind;

	/* Daemonize. */
	if ((dflg != 0) && (daemon(0, 0) != 0)) {
		perror(me);
		exit(errno);
	}

	/* Loop. */
	do {
		if (sync_lcd_brightness() != 0) {
			perror(me);
			exit(errno);
		}
		dflg && sleep(interval);
	} while (dflg);

	return 0;
}
